"""Add tracker-scoped ticket IDs

Revision ID: 237974dd94c4
Revises: None
Create Date: 2017-09-14 07:01:22.888804

"""

# revision identifiers, used by Alembic.
revision = '237974dd94c4'
down_revision = None

from alembic import op
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, Session as BaseSession, relationship

Session = sessionmaker()
Base = declarative_base()

class Ticket(Base):
    __tablename__ = 'ticket'
    id = sa.Column(sa.Integer, primary_key=True)
    scoped_id = sa.Column(sa.Integer)
    tracker_id = sa.Column(sa.Integer, sa.ForeignKey("tracker.id"), nullable=False)
    tracker = sa.orm.relationship("Tracker", backref=sa.orm.backref("tickets"))

class Tracker(Base):
    __tablename__ = 'tracker'
    id = sa.Column(sa.Integer, primary_key=True)
    next_ticket_id = sa.Column(sa.Integer, nullable=False, default=1)

def upgrade():
    bind = op.get_bind()
    session = Session(bind=bind)
    op.add_column('ticket', sa.Column('scoped_id', sa.Integer()))
    op.add_column('tracker', sa.Column('next_ticket_id',
        sa.Integer(),
        nullable=False,
        server_default='1'))
    session.commit()

    for ticket in session.query(Ticket).all():
        ticket.scoped_id = ticket.id
    session.commit()

    for tracker in session.query(Tracker).all():
        if not any(tracker.tickets):
            continue
        tracker.next_ticket_id = max(ticket.id for ticket in tracker.tickets) + 1
    session.commit()

    op.create_index(op.f('ix_ticket_scoped_id'), 'ticket', ['scoped_id'])
    op.create_unique_constraint('uq_ticket_scoped_id_tracker_id', 'ticket',
            ['scoped_id', 'tracker_id'])
    op.alter_column('ticket', 'scoped_id', nullable=False)


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('tracker', 'next_ticket_id')
    op.drop_index(op.f('ix_ticket_scoped_id'), table_name='ticket')
    op.drop_column('ticket', 'scoped_id')
    # ### end Alembic commands ###
